#pragma once

#include <queue>
#include "types.h"

namespace Nodable
{
    // forward declaration
    class MemberConnector;
    class NodeConnector;

    enum class EventType
    {
        none = 0, // operation on nodes

        delete_node_action_triggered = 0x100, // operation on nodes
        arrange_node_action_triggered,
        select_successor_node_action_triggered,
        toggle_folding_selected_node_action_triggered,

        member_connector_dropped = 0x200, // operation on member connectors
        member_connector_disconnected,

        node_connector_dropped   = 0x300, // operation on node connectors
        node_connector_disconnected
    };

    struct SimpleEvent
    {
        EventType type;
    };

    struct ToggleFoldingEvent
    {
        EventType              type;
        bool                   recursive;
    };

    struct MemberConnectorEvent
    {
        EventType              type;
        const MemberConnector* src;
        const MemberConnector* dst;
    };

    struct NodeConnectorEvent
    {
        EventType            type;
        const NodeConnector* src;
        const NodeConnector* dst;
    };

    union Event
    {
        EventType            type;
        SimpleEvent          common;
        MemberConnectorEvent member_connectors;
        NodeConnectorEvent   node_connectors;
        ToggleFoldingEvent   toggle_folding;
    };

    class EventManager
    {
        static std::queue<Event> s_events;
    public:
        static size_t poll_event(Event&);
        static void   push_event(Event&);
        static void   push_event(EventType); // to push a SimpleEvent by type
    };
}