#pragma once

#include <memory> // std::shared_ptr

#include <nodable/core/INodeFactory.h>
#include <nodable/core/Invokable.h>
#include <nodable/core/IScope.h>
#include <nodable/core/reflection/reflection>

namespace Nodable
{
    /**
     * @brief The NodeFactory instantiate Nodes. Class take a function to apply after creation.
     *
     * By default post processing function does nothing. It can be used to add a NodeView, log messages, etc.
     */
    class NodeFactory: public INodeFactory
    {
    public:
        NodeFactory(const Language* _language, std::function<void(Node *)> _post_process_fct = [](Node* _node){} )
        : m_language(_language)
        , m_post_process(_post_process_fct) {}
        ~NodeFactory() {}

        Node*                       new_program()const override ;
        InstructionNode*            new_instr()const override ;
        VariableNode*				new_variable(type, const std::string&, IScope *)const override ;
        LiteralNode*                new_literal(type)const override ;
        Node*                       new_abstract_function(const Signature*)const override;
        Node*                       new_function(const IInvokable*)const override ;
        Node*                       new_scope()const override ;
        ConditionalStructNode*      new_cond_struct()const override ;
        ForLoopNode*                new_for_loop_node()const override ;
        Node*                       new_node()const override ;
    private:
        Node*                       _new_abstract_function(const Signature*) const; // this do not invoke post_process
        void                        add_invokable_component(Node *_node, const Signature*, const IInvokable *_invokable) const;

        std::function<void(Node*)>  m_post_process; // invoked after each node creation, just before to return.
        const Language*             m_language;
    };
}
